@@ -2,6 +2,7 @@ import ipaddress
 import logging
 import os
 import ssl
+import urllib.parse
 from pathlib import Path
 from typing import Any
 from typing import Literal
@@ -17,6 +18,7 @@ from mitmproxy import certs
 from mitmproxy import connection
 from mitmproxy import ctx
 from mitmproxy import exceptions
+from mitmproxy import http
 from mitmproxy import tls
 from mitmproxy.net import tls as net_tls
 from mitmproxy.options import CONF_BASENAME
@@ -247,10 +249,8 @@ class TlsConfig:
         )
         tls_start.ssl_conn = SSL.Connection(ssl_ctx)
 
-        tls_start.ssl_conn.use_certificate(entry.cert.to_pyopenssl())
-        tls_start.ssl_conn.use_privatekey(
-            crypto.PKey.from_cryptography_key(entry.privatekey)
-        )
+        tls_start.ssl_conn.use_certificate(entry.cert.to_cryptography())
+        tls_start.ssl_conn.use_privatekey(entry.privatekey)
 
         # Force HTTP/1 for secure web proxies, we currently don't support CONNECT over HTTP/2.
         # There is a proof-of-concept branch at https://github.com/mhils/mitmproxy/tree/http2-proxy,
@@ -373,7 +373,7 @@ class TlsConfig:
             raise ValueError("Cannot validate certificate hostname without SNI")
 
         if server.alpn_offers:
-            tls_start.ssl_conn.set_alpn_protos(server.alpn_offers)
+            tls_start.ssl_conn.set_alpn_protos(list(server.alpn_offers))
 
         tls_start.ssl_conn.set_connect_state()
 
@@ -583,6 +583,9 @@ class TlsConfig:
                 f"for insecure TLS versions to work."
             )
 
+    def crl_path(self) -> str:
+        return f"/mitmproxy-{self.certstore.default_ca.serial}.crl"
+
     def get_cert(self, conn_context: context.Context) -> certs.CertStoreEntry:
         """
         This function determines the Common Name (CN), Subject Alternative Names (SANs) and Organization Name
@@ -590,16 +593,29 @@ class TlsConfig:
         """
         altnames: list[x509.GeneralName] = []
         organization: str | None = None
+        crl_distribution_point: str | None = None
 
         # Use upstream certificate if available.
         if ctx.options.upstream_cert and conn_context.server.certificate_list:
-            upstream_cert = conn_context.server.certificate_list[0]
+            upstream_cert: certs.Cert = conn_context.server.certificate_list[0]
             if upstream_cert.cn:
                 altnames.append(_ip_or_dns_name(upstream_cert.cn))
             altnames.extend(upstream_cert.altnames)
             if upstream_cert.organization:
                 organization = upstream_cert.organization
 
+            # Replace original URL path with the CA cert serial number, which acts as a magic token
+            if crls := upstream_cert.crl_distribution_points:
+                try:
+                    scheme, netloc, *_ = urllib.parse.urlsplit(crls[0])
+                except ValueError:
+                    logger.info(f"Failed to parse CRL URL: {crls[0]!r}")
+                else:
+                    # noinspection PyTypeChecker
+                    crl_distribution_point = urllib.parse.urlunsplit(
+                        (scheme, netloc, self.crl_path(), None, None)
+                    )
+
         # Add SNI or our local IP address.
         if conn_context.client.sni:
             altnames.append(_ip_or_dns_name(conn_context.client.sni))
@@ -616,7 +632,20 @@ class TlsConfig:
         # RFC 2818: If a subjectAltName extension of type dNSName is present, that MUST be used as the identity.
         # In other words, the Common Name is irrelevant then.
         cn = next((str(x.value) for x in altnames), None)
-        return self.certstore.get_cert(cn, altnames, organization)
+        return self.certstore.get_cert(
+            cn, altnames, organization, crl_distribution_point
+        )
+
+    def request(self, flow: http.HTTPFlow):
+        if not flow.live or flow.error or flow.response:
+            return
+        # Check if a request has a magic CRL token at the end
+        if flow.request.path.endswith(self.crl_path()):
+            flow.response = http.Response.make(
+                200,
+                self.certstore.default_crl,
+                {"Content-Type": "application/pkix-crl"},
+            )
 
 
 def _ip_or_dns_name(val: str) -> x509.GeneralName:
