@@ -1,5 +1,3 @@
-import logging
-import math
 import sys
 from functools import lru_cache
 
@@ -7,12 +5,14 @@ import urwid
 
 import mitmproxy.flow
 import mitmproxy.tools.console.master
+import mitmproxy_rs.syntax_highlight
 from mitmproxy import contentviews
 from mitmproxy import ctx
 from mitmproxy import dns
 from mitmproxy import http
 from mitmproxy import tcp
 from mitmproxy import udp
+from mitmproxy.dns import DNSMessage
 from mitmproxy.tools.console import common
 from mitmproxy.tools.console import flowdetailview
 from mitmproxy.tools.console import layoutwidget
@@ -51,8 +51,8 @@ class FlowDetails(tabs.Tabs):
         super().__init__([])
         self.show()
         self.last_displayed_body = None
-        contentviews.on_add.connect(self.contentview_changed)
-        contentviews.on_remove.connect(self.contentview_changed)
+        self.last_displayed_websocket_messages = None
+        contentviews.registry.on_change.connect(self.contentview_changed)
 
     @property
     def view(self):
@@ -104,7 +104,12 @@ class FlowDetails(tabs.Tabs):
                 ]
             self.show()
         else:
-            self.master.window.pop()
+            # Get the top window from the focus stack (the currently active view).
+            # If it's NOT the "flowlist", it's safe to pop back to the previous view.
+            if self.master.window.focus_stack().stack[-1] != "flowlist":
+                self.master.window.pop()
+            # If it is the "flowlist", we’re already at the main view with no flows to show.
+            # Popping now would close the last window and prompt app exit, so we remain on the empty flow list screen instead.
 
     def tab_http_request(self):
         flow = self.flow
@@ -219,15 +224,16 @@ class FlowDetails(tabs.Tabs):
 
         widget_lines = []
         for m in flow.websocket.messages:
-            _, lines, _ = contentviews.get_message_content_view(viewmode, m, flow)
-
-            for line in lines:
-                if m.from_client:
-                    line.insert(0, self.FROM_CLIENT_MARKER)
-                else:
-                    line.insert(0, self.TO_CLIENT_MARKER)
-
-                widget_lines.append(urwid.Text(line))
+            pretty = contentviews.prettify_message(m, flow, viewmode)
+            chunks = mitmproxy_rs.syntax_highlight.highlight(
+                pretty.text,
+                language=pretty.syntax_highlight,
+            )
+            if m.from_client:
+                marker = self.FROM_CLIENT_MARKER
+            else:
+                marker = self.TO_CLIENT_MARKER
+            widget_lines.append(urwid.Text([marker, *chunks]))
 
         if flow.websocket.closed_by_client is not None:
             widget_lines.append(
@@ -256,7 +262,14 @@ class FlowDetails(tabs.Tabs):
             0, self._contentview_status_bar(viewmode.capitalize(), viewmode)
         )
 
-        return searchable.Searchable(widget_lines)
+        if (last_view := self.last_displayed_websocket_messages) is not None:
+            last_view.walker[:] = widget_lines
+            view = last_view
+        else:
+            view = searchable.Searchable(widget_lines)
+            self.last_displayed_websocket_messages = view
+
+        return view
 
     def view_message_stream(self) -> urwid.Widget:
         flow = self.flow
@@ -269,15 +282,17 @@ class FlowDetails(tabs.Tabs):
 
         widget_lines = []
         for m in flow.messages:
-            _, lines, _ = contentviews.get_message_content_view(viewmode, m, flow)
+            if m.from_client:
+                marker = self.FROM_CLIENT_MARKER
+            else:
+                marker = self.TO_CLIENT_MARKER
+            pretty = contentviews.prettify_message(m, flow, viewmode)
+            chunks = mitmproxy_rs.syntax_highlight.highlight(
+                pretty.text,
+                language=pretty.syntax_highlight,
+            )
 
-            for line in lines:
-                if m.from_client:
-                    line.insert(0, self.FROM_CLIENT_MARKER)
-                else:
-                    line.insert(0, self.TO_CLIENT_MARKER)
-
-                widget_lines.append(urwid.Text(line))
+            widget_lines.append(urwid.Text([marker, *chunks]))
 
         if flow.intercepted:
             markup = widget_lines[-1].get_text()[0]
@@ -292,10 +307,16 @@ class FlowDetails(tabs.Tabs):
     def view_details(self):
         return flowdetailview.flowdetails(self.view, self.flow)
 
-    def content_view(self, viewmode, message):
+    def content_view(
+        self, viewmode: str, message: http.Message
+    ) -> tuple[str, list[urwid.Text]]:
         if message.raw_content is None:
-            msg, body = "", [urwid.Text([("error", "[content missing]")])]
-            return msg, body
+            return "", [urwid.Text([("error", "[content missing]")])]
+        elif message.raw_content == b"":
+            if isinstance(message, http.Request):
+                return "", [urwid.Text("No request content")]
+            else:
+                return "", [urwid.Text("No content")]
         else:
             full = self.master.commands.execute(
                 "view.settings.getval @focus fullcontents false"
@@ -319,54 +340,37 @@ class FlowDetails(tabs.Tabs):
             )
 
     @lru_cache(maxsize=200)
-    def _get_content_view(self, viewmode, max_lines, _):
-        message = self._get_content_view_message
-        self._get_content_view_message = None
-        description, lines, error = contentviews.get_message_content_view(
-            viewmode, message, self.flow
+    def _get_content_view(
+        self, viewmode: str, max_lines: int, _
+    ) -> tuple[str, list[urwid.Text]]:
+        message: http.Message = self._get_content_view_message
+        self._get_content_view_message = None  # type: ignore[assignment]
+
+        pretty = contentviews.prettify_message(message, self.flow, viewmode)
+        cut_off = strutils.cut_after_n_lines(pretty.text, max_lines)
+
+        chunks = mitmproxy_rs.syntax_highlight.highlight(
+            cut_off,
+            language=pretty.syntax_highlight,
         )
-        if error:
-            logging.debug(error)
-        # Give hint that you have to tab for the response.
-        if description == "No content" and isinstance(message, http.Request):
-            description = "No request content"
 
-        # If the users has a wide terminal, he gets fewer lines; this should not be an issue.
-        chars_per_line = 80
-        max_chars = max_lines * chars_per_line
-        total_chars = 0
-        text_objects = []
-        for line in lines:
-            txt = []
-            for style, text in line:
-                if total_chars + len(text) > max_chars:
-                    text = text[: max_chars - total_chars]
-                txt.append((style, text))
-                total_chars += len(text)
-                if total_chars == max_chars:
-                    break
-
-            # round up to the next line.
-            total_chars = int(math.ceil(total_chars / chars_per_line) * chars_per_line)
-
-            text_objects.append(urwid.Text(txt))
-            if total_chars == max_chars:
-                text_objects.append(
-                    urwid.Text(
-                        [
-                            (
-                                "highlight",
-                                "Stopped displaying data after %d lines. Press "
-                                % max_lines,
-                            ),
-                            ("key", "f"),
-                            ("highlight", " to load all data."),
-                        ]
-                    )
+        text_objects = [urwid.Text(chunks)]
+        if len(cut_off) < len(pretty.text):
+            text_objects.append(
+                urwid.Text(
+                    [
+                        (
+                            "highlight",
+                            "Stopped displaying data after %d lines. Press "
+                            % max_lines,
+                        ),
+                        ("key", "f"),
+                        ("highlight", " to load all data."),
+                    ]
                 )
-                break
+            )
 
-        return description, text_objects
+        return f"{pretty.view_name} {pretty.description}", text_objects
 
     def conn_text(self, conn):
         if conn:
@@ -396,25 +400,7 @@ class FlowDetails(tabs.Tabs):
             viewmode = self.master.commands.call("console.flowview.mode")
             msg, body = self.content_view(viewmode, conn)
 
-            cols = [
-                urwid.Text(
-                    [
-                        ("heading", msg),
-                    ]
-                ),
-                urwid.Text(
-                    [
-                        " ",
-                        ("heading", "["),
-                        ("heading_key", "m"),
-                        ("heading", (":%s]" % viewmode)),
-                    ],
-                    align="right",
-                ),
-            ]
-            title = urwid.AttrMap(urwid.Columns(cols), "heading")
-
-            txt.append(title)
+            txt.append(self._contentview_status_bar(msg, viewmode))
             txt.extend(body)
         else:
             txt = [
@@ -430,14 +416,32 @@ class FlowDetails(tabs.Tabs):
         return searchable.Searchable(txt)
 
     def dns_message_text(
-        self, type: str, message: dns.Message | None
+        self, type: str, message: DNSMessage | None
     ) -> searchable.Searchable:
+        """
+        Alternative:
+        if not message:
+            return searchable.Searchable([urwid.Text(("highlight", f"No {typ}."))])
+
+        viewmode = self.master.commands.call("console.flowview.mode")
+        pretty = contentviews.prettify_message(message, flow, viewmode)
+        chunks = mitmproxy_rs.syntax_highlight.highlight(
+            pretty.text,
+            language=pretty.syntax_highlight,
+        )
+
+        widget_lines = [
+            self._contentview_status_bar(viewmode.capitalize(), viewmode),
+            urwid.Text(chunks)
+        ]
+        return searchable.Searchable(widget_lines)
+        """
         # Keep in sync with web/src/js/components/FlowView/DnsMessages.tsx
         if message:
 
             def rr_text(rr: dns.ResourceRecord):
                 return urwid.Text(
-                    f"  {rr.name} {dns.types.to_str(rr.type)} {dns.classes.to_str(rr.class_)} {rr.ttl} {str(rr)}"
+                    f"  {rr.name} {dns.types.to_str(rr.type)} {dns.classes.to_str(rr.class_)} {rr.ttl} {rr}"
                 )
 
             txt = []
